package sun.print;

import javax.print.attribute.standard.MediaSizeName;
import javax.print.attribute.standard.Media;
import javax.print.attribute.standard.Copies;
import javax.print.attribute.standard.Fidelity;
import javax.print.attribute.DocAttributeSet;
import javax.print.attribute.Attribute;
import javax.print.attribute.standard.DocumentName;
import javax.print.attribute.standard.JobName;
import javax.print.attribute.standard.RequestingUserName;
import javax.print.attribute.standard.JobOriginatingUserName;
import javax.print.attribute.PrintRequestAttribute;
import javax.print.attribute.PrintJobAttribute;
import javax.print.attribute.HashPrintRequestAttributeSet;
import java.awt.print.PrinterException;
import java.awt.print.Paper;
import java.awt.print.PageFormat;
import java.net.URL;
import java.awt.print.Pageable;
import java.awt.print.Printable;
import javax.print.DocFlavor;
import javax.print.PrintException;
import javax.print.event.PrintJobAttributeListener;
import javax.print.event.PrintJobEvent;
import java.io.IOException;
import javax.print.event.PrintJobListener;
import javax.print.attribute.AttributeSetUtilities;
import javax.print.attribute.HashPrintJobAttributeSet;
import javax.print.PrintService;
import javax.print.attribute.standard.OrientationRequested;
import javax.print.attribute.standard.MediaSize;
import java.io.Reader;
import java.io.InputStream;
import javax.print.Doc;
import java.awt.print.PrinterJob;
import javax.print.attribute.PrintJobAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;
import java.util.Vector;
import javax.print.CancelablePrintJob;

public class PSStreamPrintJob implements CancelablePrintJob
{
    private transient Vector jobListeners;
    private transient Vector attrListeners;
    private transient Vector listenedAttributeSets;
    private PSStreamPrintService service;
    private boolean fidelity;
    private boolean printing;
    private boolean printReturned;
    private PrintRequestAttributeSet reqAttrSet;
    private PrintJobAttributeSet jobAttrSet;
    private PrinterJob job;
    private Doc doc;
    private InputStream instream;
    private Reader reader;
    private String jobName;
    private int copies;
    private MediaSize mediaSize;
    private OrientationRequested orient;
    
    PSStreamPrintJob(PSStreamPrintService service) {
        this.printing = false;
        this.printReturned = false;
        this.reqAttrSet = null;
        this.jobAttrSet = null;
        this.instream = null;
        this.reader = null;
        this.jobName = "Java Printing";
        this.copies = 1;
        this.mediaSize = MediaSize.NA.LETTER;
        this.orient = OrientationRequested.PORTRAIT;
        this.service = service;
    }
    
    @Override
    public PrintService getPrintService() {
        return this.service;
    }
    
    @Override
    public PrintJobAttributeSet getAttributes() {
        synchronized (this) {
            if (this.jobAttrSet == null) {
                return AttributeSetUtilities.unmodifiableView(new HashPrintJobAttributeSet());
            }
            return this.jobAttrSet;
        }
    }
    
    @Override
    public void addPrintJobListener(PrintJobListener e) {
        synchronized (this) {
            if (e == null) {
                return;
            }
            if (this.jobListeners == null) {
                this.jobListeners = new Vector();
            }
            this.jobListeners.add(e);
        }
    }
    
    @Override
    public void removePrintJobListener(PrintJobListener o) {
        synchronized (this) {
            if (o == null || this.jobListeners == null) {
                return;
            }
            this.jobListeners.remove(o);
            if (this.jobListeners.isEmpty()) {
                this.jobListeners = null;
            }
        }
    }
    
    private void closeDataStreams() {
        if (this.doc == null) {
            return;
        }
        Object printData;
        try {
            printData = this.doc.getPrintData();
        }
        catch (IOException ex) {
            return;
        }
        if (this.instream != null) {
            try {
                this.instream.close();
            }
            catch (IOException ex5) {}
            finally {
                this.instream = null;
            }
        }
        else if (this.reader != null) {
            try {
                this.reader.close();
            }
            catch (IOException ex4) {}
            finally {
                this.reader = null;
            }
        }
        else if ((printData instanceof InputStream)) {
            try {
                ((InputStream)printData).close();
            }
            catch (IOException ex3) {}
        }
        else if ((printData instanceof Reader)) {
            try {
                ((Reader)printData).close();
            }
            catch (IOException ex2) {}
        }
        return;
    }
    
    private void notifyEvent(int reason) {
        synchronized (this) {
            if (this.jobListeners != null) {
                PrintJobEvent printJobEvent = new PrintJobEvent(this, reason);
                for (int i = 0; i < this.jobListeners.size(); ++i) {
                    PrintJobListener printJobListener = (PrintJobListener)(PrintJobListener)this.jobListeners.elementAt(i);
                    switch (reason) {
                        case 101: {
                            printJobListener.printJobCanceled(printJobEvent);
                            break;
                        }
                        case 103: {
                            printJobListener.printJobFailed(printJobEvent);
                            break;
                        }
                        case 106: {
                            printJobListener.printDataTransferCompleted(printJobEvent);
                            break;
                        }
                        case 105: {
                            printJobListener.printJobNoMoreEvents(printJobEvent);
                            break;
                        }
                        case 102: {
                            printJobListener.printJobCompleted(printJobEvent);
                            break;
                        }
                    }
                }
            }
        }
    }
    
    @Override
    public void addPrintJobAttributeListener(PrintJobAttributeListener e, PrintJobAttributeSet e2) {
        synchronized (this) {
            if (e == null) {
                return;
            }
            if (this.attrListeners == null) {
                this.attrListeners = new Vector();
                this.listenedAttributeSets = new Vector();
            }
            this.attrListeners.add(e);
            if (e2 == null) {
                e2 = new HashPrintJobAttributeSet();
            }
            this.listenedAttributeSets.add(e2);
        }
    }
    
    @Override
    public void removePrintJobAttributeListener(PrintJobAttributeListener o) {
        synchronized (this) {
            if (o == null || this.attrListeners == null) {
                return;
            }
            int index = this.attrListeners.indexOf(o);
            if (index == -1) {
                return;
            }
            this.attrListeners.remove(index);
            this.listenedAttributeSets.remove(index);
            if (this.attrListeners.isEmpty()) {
                this.attrListeners = null;
                this.listenedAttributeSets = null;
            }
        }
    }
    
    @Override
    public void print(Doc doc, PrintRequestAttributeSet set) throws PrintException {
        synchronized (this) {
            if (this.printing) {
                throw new PrintException("already printing");
            }
            this.printing = true;
        }
        this.doc = doc;
        DocFlavor docFlavor = doc.getDocFlavor();
        Object printData;
        try {
            printData = doc.getPrintData();
        }
        catch (IOException ex) {
            this.notifyEvent(103);
            throw new PrintException("can't get print data: " + ex.toString());
        }
        if (docFlavor == null || !this.service.isDocFlavorSupported(docFlavor)) {
            this.notifyEvent(103);
            throw new PrintJobFlavorException("invalid flavor", docFlavor);
        }
        this.initializeAttributeSets(doc, set);
        this.getAttributeValues(docFlavor);
        String representationClassName = docFlavor.getRepresentationClassName();
        if (!docFlavor.equals(DocFlavor.INPUT_STREAM.GIF) && !docFlavor.equals(DocFlavor.INPUT_STREAM.JPEG) && !docFlavor.equals(DocFlavor.INPUT_STREAM.PNG) && !docFlavor.equals(DocFlavor.BYTE_ARRAY.GIF) && !docFlavor.equals(DocFlavor.BYTE_ARRAY.JPEG) && !docFlavor.equals(DocFlavor.BYTE_ARRAY.PNG)) {
            if (!docFlavor.equals(DocFlavor.URL.GIF) && !docFlavor.equals(DocFlavor.URL.JPEG) && !docFlavor.equals(DocFlavor.URL.PNG)) {
                if (representationClassName.equals("java.awt.print.Pageable")) {
                    try {
                        this.pageableJob((Pageable)doc.getPrintData(), this.reqAttrSet);
                        return;
                    }
                    catch (ClassCastException e3) {
                        this.notifyEvent(103);
                        throw new PrintException(e3);
                    }
                    catch (IOException e4) {
                        this.notifyEvent(103);
                        throw new PrintException(e4);
                    }
                }
                else if (representationClassName.equals("java.awt.print.Printable")) {
                    try {
                        this.printableJob((Printable)doc.getPrintData(), this.reqAttrSet);
                        return;
                    }
                    catch (ClassCastException e) {
                        this.notifyEvent(103);
                        throw new PrintException(e);
                    }
                    catch (IOException e2) {
                        this.notifyEvent(103);
                        throw new PrintException(e2);
                    }
                }
                else {
                    this.notifyEvent(103);
                    throw new PrintException("unrecognized class: " + representationClassName);
                }
            }
            try {
                this.printableJob(new ImagePrinter((URL)printData), this.reqAttrSet);
                return;
            }
            catch (ClassCastException e5) {
                this.notifyEvent(103);
                throw new PrintException(e5);
            }
        }
        try {
            this.instream = doc.getStreamForBytes();
            this.printableJob(new ImagePrinter(this.instream), this.reqAttrSet);
            return;
        }
        catch (ClassCastException e6) {
            this.notifyEvent(103);
            throw new PrintException(e6);
        }
        catch (IOException e7) {
            this.notifyEvent(103);
            throw new PrintException(e7);
        }
    }
    
    public void printableJob(Printable printable, PrintRequestAttributeSet attributes) throws PrintException {
        try {
            synchronized (this) {
                if (this.job != null) {
                    throw new PrintException("already printing");
                }
                this.job = new PSPrinterJob();
            }
            this.job.setPrintService(this.getPrintService());
            PageFormat pageFormat = new PageFormat();
            if (this.mediaSize != null) {
                Paper paper = new Paper();
                paper.setSize((double)this.mediaSize.getX(25400) * 72.0, (double)this.mediaSize.getY(25400) * 72.0);
                paper.setImageableArea(72.0, 72.0, paper.getWidth() - 144.0, paper.getHeight() - 144.0);
                pageFormat.setPaper(paper);
            }
            if (this.orient == OrientationRequested.REVERSE_LANDSCAPE) {
                pageFormat.setOrientation(2);
            }
            else if (this.orient == OrientationRequested.LANDSCAPE) {
                pageFormat.setOrientation(0);
            }
            this.job.setPrintable(printable, pageFormat);
            this.job.print(attributes);
            this.notifyEvent(102);
            return;
        }
        catch (PrinterException e) {
            this.notifyEvent(103);
            throw new PrintException(e);
        }
        finally {
            this.printReturned = true;
        }
    }
    
    public void pageableJob(Pageable pageable, PrintRequestAttributeSet attributes) throws PrintException {
        try {
            synchronized (this) {
                if (this.job != null) {
                    throw new PrintException("already printing");
                }
                this.job = new PSPrinterJob();
            }
            this.job.setPrintService(this.getPrintService());
            this.job.setPageable(pageable);
            this.job.print(attributes);
            this.notifyEvent(102);
            return;
        }
        catch (PrinterException e) {
            this.notifyEvent(103);
            throw new PrintException(e);
        }
        finally {
            this.printReturned = true;
        }
    }
    
    private synchronized void initializeAttributeSets(Doc obj, PrintRequestAttributeSet set) {
        this.reqAttrSet = new HashPrintRequestAttributeSet();
        this.jobAttrSet = new HashPrintJobAttributeSet();
        if (set != null) {
            this.reqAttrSet.addAll(set);
            Attribute[] array = set.toArray();
            for (int i = 0; i < array.length; ++i) {
                if ((array[i] instanceof PrintJobAttribute)) {
                    this.jobAttrSet.add(array[i]);
                }
            }
        }
        DocAttributeSet attributes = obj.getAttributes();
        if (attributes != null) {
            Attribute[] array2 = attributes.toArray();
            for (int j = 0; j < array2.length; ++j) {
                if ((array2[j] instanceof PrintRequestAttribute)) {
                    this.reqAttrSet.add(array2[j]);
                }
                if ((array2[j] instanceof PrintJobAttribute)) {
                    this.jobAttrSet.add(array2[j]);
                }
            }
        }
        String property = "";
        try {
            property = System.getProperty("user.name");
        }
        catch (SecurityException ex) {}
        if (property == null || property.equals("")) {
            RequestingUserName requestingUserName = (RequestingUserName)set.get(RequestingUserName.class);
            if (requestingUserName != null) {
                this.jobAttrSet.add(new JobOriginatingUserName(requestingUserName.getValue(), requestingUserName.getLocale()));
            }
            else {
                this.jobAttrSet.add(new JobOriginatingUserName("", null));
            }
        }
        else {
            this.jobAttrSet.add(new JobOriginatingUserName(property, null));
        }
        if (this.jobAttrSet.get(JobName.class) == null) {
            if (attributes != null && attributes.get(DocumentName.class) != null) {
                DocumentName documentName = (DocumentName)attributes.get(DocumentName.class);
                this.jobAttrSet.add(new JobName(documentName.getValue(), documentName.getLocale()));
            }
            else {
                String jobName = "JPS Job:" + obj;
                try {
                    if ((obj.getPrintData() instanceof URL)) {
                        jobName = ((URL)(URL)obj.getPrintData()).toString();
                    }
                }
                catch (IOException ex2) {}
                this.jobAttrSet.add(new JobName(jobName, null));
            }
        }
        this.jobAttrSet = AttributeSetUtilities.unmodifiableView(this.jobAttrSet);
    }
    
    private void getAttributeValues(DocFlavor docFlavor) throws PrintException {
        if (this.reqAttrSet.get(Fidelity.class) == Fidelity.FIDELITY_TRUE) {
            this.fidelity = true;
        }
        else {
            this.fidelity = false;
        }
        Attribute[] array = this.reqAttrSet.toArray();
        for (int i = 0; i < array.length; ++i) {
            Attribute obj = array[i];
            Class<Object> category = obj.getCategory();
            if (this.fidelity) {
                if (!this.service.isAttributeCategorySupported(category)) {
                    this.notifyEvent(103);
                    throw new PrintJobAttributeException("unsupported category: " + category, category, null);
                }
                if (!this.service.isAttributeValueSupported(obj, docFlavor, null)) {
                    this.notifyEvent(103);
                    throw new PrintJobAttributeException("unsupported attribute: " + obj, null, obj);
                }
            }
            if (category == JobName.class) {
                this.jobName = ((JobName)obj).getValue();
            }
            else if (category == Copies.class) {
                this.copies = ((Copies)obj).getValue();
            }
            else if (category == Media.class) {
                if (obj instanceof MediaSizeName && this.service.isAttributeValueSupported(obj, null, null)) {
                    this.mediaSize = MediaSize.getMediaSizeForName((MediaSizeName)obj);
                }
            }
            else if (category == OrientationRequested.class) {
                this.orient = (OrientationRequested)obj;
            }
        }
    }
    
    @Override
    public void cancel() throws PrintException {
        synchronized (this) {
            if (!this.printing) {
                throw new PrintException("Job is not yet submitted.");
            }
            if (this.job != null && !this.printReturned) {
                this.job.cancel();
                this.notifyEvent(101);
                return;
            }
            throw new PrintException("Job could not be cancelled.");
        }
    }
}
